/*
 * Main.java
 *
 * Created on 29 settembre 2007, 8.34
 */

package it.jplasmide.gui;

import it.jplasmide.gui.listeners.BracketOperationsListener;
import it.jplasmide.gui.utils.BackupManager;
import it.jplasmide.gui.utils.PlasmGuiOperations;
import it.jplasmide.utils.PlasmEventListener;
import it.jplasmide.utils.PlasmWrapper;
import it.jplasmide.doctool.DataBeanContainer;
import it.jplasmide.doctool.DataBeanExporter;
import it.jplasmide.doctool.XMLProcessor;
import it.jplasmide.gui.utils.PlasmPane;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.net.URL;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JTextPane;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.SimpleAttributeSet;
import org.omg.PortableServer.LIFESPAN_POLICY_ID;

/**
 * TODO List:
 * 
 *  - bugfixing
 *  - log su file
 *  - inc/dec editor font size DONE
 *  - inc/dec console font size DONE
 *  - test with macosx
 *  - display caret line number DONE
 *  - more features to configure
 *  
 * @author  Dario Lucia
 *
 */

public class Main extends javax.swing.JFrame implements PlasmEventListener,CaretListener {
    
    private static final String[] DOCS_FILES = new String[] { "plasm_doc.xml" };       
    private static final int BACKUP_SECONDS = 120;
    
    private String mzschemePath = null;
    private String plasmPath = null;
    
    private ConfigureForm configForm;
    
    private JFileChooser saveDialog = new JFileChooser();
    private JFileChooser openDialog = new JFileChooser();
    
    private String NEW_LINE = System.getProperty("line.separator");
    
    /* Tab popup menu for ops: cut, copy, paste, eval selection, view selection  - Lazy init when requested */
    private JPopupMenu tabMenu = null;


    private PlasmWrapper plasm;
    
    private DataBeanExporter htmlExporter;
    private DataBeanContainer dataBeanContainer;
    
    private int editorFontSize = 12;
    private int consoleFontSize = 10;
    
    private int currentFindPosition = 0;
    
    private PlasmGuiOperations ops;
    private BackupManager backup; 

    private Set<String> expObj;
    private Set<String> expFunc;
    
    private int divLocation;
    /** Creates new form Main */
    public Main() {
        initComponents();
        
        /* Set console font */
        this.jTextArea1.setFont(this.jTextArea1.getFont().deriveFont(1,this.consoleFontSize));
        
        this.configForm  = new ConfigureForm();
        this.plasm = new PlasmWrapper();
        this.plasm.addPlasmListener(this);
        this.htmlExporter = new DataBeanExporter();
        this.dataBeanContainer = new DataBeanContainer();
        
        
        writeToConsole( "##################################" + NEW_LINE +
                        "##    Welcome to JPlasm 5 IDE   ##" + NEW_LINE +
                        "##        Version 1.1.0.8       ##" + NEW_LINE +
                        "##         -BETA VERSION-       ##" + NEW_LINE +
                        "##     Bugs and suggestions:    ##" + NEW_LINE +
                        "##     dario.lucia@gmail.com    ##" + NEW_LINE +
                        "##################################");
        this.setIconImage((new ImageIcon(ClassLoader.getSystemResource("3d.png"))).getImage());
     
        
        this.jTextField1.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent documentEvent) {
                findInEditorPane();
            }
            public void insertUpdate(DocumentEvent documentEvent) {
                findInEditorPane();
            }
            public void removeUpdate(DocumentEvent documentEvent) {
                findInEditorPane();
            }
        });
        
        /* Loading references doc */
        XMLProcessor proc = new XMLProcessor();
        for(String f : DOCS_FILES) {
            URL path = ClassLoader.getSystemResource(f);
            if(path != null) {
                this.dataBeanContainer.addAllBeans(proc.processDocSource(path));
            }
        }
        
        /* Init find text field */
        this.search_text.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent documentEvent) {
                resetSearchIndex();
            }
            public void insertUpdate(DocumentEvent documentEvent) {
                resetSearchIndex();
            }
            public void removeUpdate(DocumentEvent documentEvent) {
                resetSearchIndex();
            }
        });
        
        /* Update references panel once */
        findInEditorPane();
        
        /* Init Gui ops */
        this.ops = new PlasmGuiOperations(this.jTabbedPane1 , this.dataBeanContainer);
        
        /* Set memo for exported objects and functions */
        this.expObj = new TreeSet<String>();
        this.expFunc = new TreeSet<String>();
        
        /* Init backup */
        this.backup = new BackupManager(BACKUP_SECONDS,this.jTabbedPane1);
        this.backup.start();
    }
   
    protected void resetSearchIndex() {
        this.currentFindPosition = 0;
    }   
    
    protected void findInEditorPane() {
        String toFind = this.jTextField1.getText();
        String jeditorText = this.htmlExporter.exportToHTML(this.dataBeanContainer.getBeans(toFind));
        this.jEditorPane1.setText(jeditorText);
        this.jEditorPane1.setCaretPosition(0);
    }
    
    /** 
     * Code and events for components
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jToolBar1 = new javax.swing.JToolBar();
        newFile_button = new javax.swing.JButton();
        openFile_button = new javax.swing.JButton();
        saveFile_button = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        closeFile_button = new javax.swing.JButton();
        undo_button = new javax.swing.JButton();
        redo_button = new javax.swing.JButton();
        cut_button = new javax.swing.JButton();
        copy_button = new javax.swing.JButton();
        paste_button = new javax.swing.JButton();
        startPlasm_button = new javax.swing.JButton();
        stopPlasm_button = new javax.swing.JButton();
        evalSel_button = new javax.swing.JButton();
        evalAll_button = new javax.swing.JButton();
        evalSelScheme_button = new javax.swing.JButton();
        evalAllScheme_button = new javax.swing.JButton();
        viewOpenGL_button = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jToggleButton1 = new javax.swing.JToggleButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel4 = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jSplitPane2 = new javax.swing.JSplitPane();
        consolePanel = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        jPanel3 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        referencePanel = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        jEditorPane1 = new javax.swing.JEditorPane();
        findPanel = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        search_text = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();
        replace_text = new javax.swing.JTextField();
        next_button1 = new javax.swing.JButton();
        replace_button = new javax.swing.JButton();
        replaceall_button = new javax.swing.JButton();
        objsPanel = new javax.swing.JPanel();
        jLabel9 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        lastObj_list = new javax.swing.JList();
        viewObj_button = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        lastFunc_list = new javax.swing.JList();
        jPanel6 = new javax.swing.JPanel();
        showHelp_button = new javax.swing.JButton();
        defObj_button = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jToolBar2 = new javax.swing.JToolBar();
        jPanel2 = new javax.swing.JPanel();
        caretLine_label = new javax.swing.JLabel();
        saving_label = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        sendingCommand_label = new javax.swing.JLabel();
        working_label = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        file_dropmenu = new javax.swing.JMenu();
        newFile_menu = new javax.swing.JMenuItem();
        openFile_menu = new javax.swing.JMenuItem();
        saveFile_menu = new javax.swing.JMenuItem();
        saveAs_menu = new javax.swing.JMenuItem();
        saveAll_menu = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        closeFile_menu = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        quit_menu = new javax.swing.JMenuItem();
        modify_dropmenu = new javax.swing.JMenu();
        undo_menu = new javax.swing.JMenuItem();
        redo_menu = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        cut_menu = new javax.swing.JMenuItem();
        copy_menu = new javax.swing.JMenuItem();
        paste_menu = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        tilde_menu = new javax.swing.JMenuItem();
        jSeparator9 = new javax.swing.JSeparator();
        autoclosebrackets_menu = new javax.swing.JCheckBoxMenuItem();
        jSeparator10 = new javax.swing.JSeparator();
        incEditFont_menu = new javax.swing.JMenuItem();
        decEditFont_menu = new javax.swing.JMenuItem();
        plasm_dropmenu = new javax.swing.JMenu();
        startProcess_menu = new javax.swing.JMenuItem();
        stopProcess_menu = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        clearConsole_menu = new javax.swing.JMenuItem();
        incConsoleFont_menu = new javax.swing.JMenuItem();
        decConsoleFont_menu = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        evalSelection_menu = new javax.swing.JMenuItem();
        evalFile_menu = new javax.swing.JMenuItem();
        view_menu = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JSeparator();
        evalSelScheme_menu = new javax.swing.JMenuItem();
        evalFileScheme_menu = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JSeparator();
        brokenPipe_menu = new javax.swing.JCheckBoxMenuItem();
        configure_menu = new javax.swing.JMenuItem();
        about_dropmenu = new javax.swing.JMenu();
        aboutPlasm_menu = new javax.swing.JMenuItem();
        aboutPlasmIde_menu = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("JPlaSM 5 IDE []");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                windowClosingHandler(evt);
            }
        });

        jToolBar1.setFloatable(false);
        newFile_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/filenew.png")));
        newFile_button.setToolTipText("Create a new empty file");
        newFile_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newFile_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(newFile_button);

        openFile_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/fileopen.png")));
        openFile_button.setToolTipText("Open file");
        openFile_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openFile_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(openFile_button);

        saveFile_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/filesave.png")));
        saveFile_button.setToolTipText("Save current tab");
        saveFile_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveFile_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(saveFile_button);

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/filesaveas.png")));
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jToolBar1.add(jButton1);

        closeFile_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/fileclose.png")));
        closeFile_button.setToolTipText("Close current tab");
        closeFile_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeFile_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(closeFile_button);

        undo_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/undo.png")));
        undo_button.setToolTipText("Undo");
        undo_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                undo_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(undo_button);

        redo_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/redo.png")));
        redo_button.setToolTipText("Redo");
        redo_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                redo_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(redo_button);

        cut_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/editcut.png")));
        cut_button.setToolTipText("Cut");
        cut_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cut_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(cut_button);

        copy_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/editcopy.png")));
        copy_button.setToolTipText("Copy");
        copy_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                copy_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(copy_button);

        paste_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/editpaste.png")));
        paste_button.setToolTipText("Paste");
        paste_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                paste_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(paste_button);

        startPlasm_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/run.png")));
        startPlasm_button.setToolTipText("Start Plasm interpreter");
        startPlasm_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startPlasm_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(startPlasm_button);

        stopPlasm_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/stop.png")));
        stopPlasm_button.setToolTipText("Stop Plasm interpreter");
        stopPlasm_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopPlasm_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(stopPlasm_button);

        evalSel_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/1downarrow.png")));
        evalSel_button.setToolTipText("Eval selection as Plasm command");
        evalSel_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalSel_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(evalSel_button);

        evalAll_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/2downarrow.png")));
        evalAll_button.setToolTipText("Eval all as Plasm command");
        evalAll_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalAll_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(evalAll_button);

        evalSelScheme_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/1uparrow.png")));
        evalSelScheme_button.setToolTipText("Eval selection as Scheme command");
        evalSelScheme_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalSelScheme_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(evalSelScheme_button);

        evalAllScheme_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/2uparrow.png")));
        evalAllScheme_button.setToolTipText("Eval all as Scheme command");
        evalAllScheme_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalAllScheme_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(evalAllScheme_button);

        viewOpenGL_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/viewmag.png")));
        viewOpenGL_button.setToolTipText("View selection in OpenGL");
        viewOpenGL_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewOpenGL_buttonActionPerformed(evt);
            }
        });

        jToolBar1.add(viewOpenGL_button);

        jToggleButton1.setFont(new java.awt.Font("Verdana", 1, 10));
        jToggleButton1.setText("Hide Panel");
        jToggleButton1.setMaximumSize(new java.awt.Dimension(72, 28));
        jToggleButton1.setMinimumSize(new java.awt.Dimension(72, 28));
        jToggleButton1.setPreferredSize(new java.awt.Dimension(72, 28));
        jToggleButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap(402, Short.MAX_VALUE)
                .addComponent(jToggleButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToggleButton1, javax.swing.GroupLayout.DEFAULT_SIZE, 34, Short.MAX_VALUE)
        );
        jToolBar1.add(jPanel1);

        jSplitPane1.setDividerLocation(600);
        jSplitPane1.setDividerSize(6);
        jTabbedPane1.setMinimumSize(new java.awt.Dimension(100, 7));
        jTabbedPane1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTabbedPane1StateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 599, Short.MAX_VALUE)
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 726, Short.MAX_VALUE)
        );
        jSplitPane1.setLeftComponent(jPanel4);

        jSplitPane2.setDividerLocation(200);
        jSplitPane2.setDividerSize(6);
        jSplitPane2.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jTextArea1.setBackground(new java.awt.Color(0, 0, 0));
        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setFont(new java.awt.Font("Monospaced", 1, 11));
        jTextArea1.setForeground(new java.awt.Color(255, 255, 255));
        jTextArea1.setRows(5);
        jTextArea1.setToolTipText("PLaSM Console");
        consolePanel.setViewportView(jTextArea1);

        jSplitPane2.setTopComponent(consolePanel);

        jPanel3.setLayout(new java.awt.BorderLayout());

        jPanel5.setLayout(new java.awt.CardLayout());

        jLabel3.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel3.setText("PLaSM Reference");

        jLabel4.setFont(new java.awt.Font("Verdana", 1, 10));
        jLabel4.setText("Find definition ");

        jEditorPane1.setEditable(false);
        jEditorPane1.setContentType("text/html");
        jScrollPane2.setViewportView(jEditorPane1);

        javax.swing.GroupLayout referencePanelLayout = new javax.swing.GroupLayout(referencePanel);
        referencePanel.setLayout(referencePanelLayout);
        referencePanelLayout.setHorizontalGroup(
            referencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, referencePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(referencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(jLabel3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, referencePanelLayout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addGap(2, 2, 2)
                        .addComponent(jTextField1, javax.swing.GroupLayout.DEFAULT_SIZE, 263, Short.MAX_VALUE)))
                .addContainerGap())
        );
        referencePanelLayout.setVerticalGroup(
            referencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(referencePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(referencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 424, Short.MAX_VALUE))
        );
        jPanel5.add(referencePanel, "plasmref");

        jLabel8.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel8.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel8.setText("Search and Replace");

        jLabel10.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel10.setText("Search:");

        search_text.setToolTipText("Text to find in the current document");
        search_text.setMaximumSize(new java.awt.Dimension(214, 26));
        search_text.setMinimumSize(new java.awt.Dimension(4, 20));

        jLabel11.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel11.setText("Replace:");

        replace_text.setToolTipText("Text to replace in the current document");
        replace_text.setMaximumSize(new java.awt.Dimension(214, 26));
        replace_text.setMinimumSize(new java.awt.Dimension(4, 20));

        next_button1.setFont(new java.awt.Font("Verdana", 1, 10));
        next_button1.setText("Find");
        next_button1.setMaximumSize(new java.awt.Dimension(40, 26));
        next_button1.setMinimumSize(new java.awt.Dimension(40, 26));
        next_button1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                next_button1ActionPerformed(evt);
            }
        });

        replace_button.setFont(new java.awt.Font("Verdana", 1, 10));
        replace_button.setText("Replace");
        replace_button.setMaximumSize(new java.awt.Dimension(40, 26));
        replace_button.setMinimumSize(new java.awt.Dimension(40, 26));
        replace_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                replace_buttonActionPerformed(evt);
            }
        });

        replaceall_button.setFont(new java.awt.Font("Verdana", 1, 10));
        replaceall_button.setText("Replace all");
        replaceall_button.setMaximumSize(new java.awt.Dimension(40, 26));
        replaceall_button.setMinimumSize(new java.awt.Dimension(40, 26));
        replaceall_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                replaceall_buttonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout findPanelLayout = new javax.swing.GroupLayout(findPanel);
        findPanel.setLayout(findPanelLayout);
        findPanelLayout.setHorizontalGroup(
            findPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(findPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(findPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel8, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addGroup(findPanelLayout.createSequentialGroup()
                        .addComponent(jLabel10, javax.swing.GroupLayout.PREFERRED_SIZE, 69, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(search_text, javax.swing.GroupLayout.DEFAULT_SIZE, 268, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, findPanelLayout.createSequentialGroup()
                        .addComponent(jLabel11, javax.swing.GroupLayout.PREFERRED_SIZE, 69, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(replace_text, javax.swing.GroupLayout.DEFAULT_SIZE, 268, Short.MAX_VALUE))
                    .addComponent(next_button1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(replace_button, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(replaceall_button, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE))
                .addContainerGap())
        );
        findPanelLayout.setVerticalGroup(
            findPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(findPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel8)
                .addGap(18, 18, 18)
                .addGroup(findPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel10)
                    .addComponent(search_text, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(23, 23, 23)
                .addGroup(findPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(replace_text, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel11))
                .addGap(20, 20, 20)
                .addComponent(next_button1, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(replace_button, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(replaceall_button, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(266, Short.MAX_VALUE))
        );
        jPanel5.add(findPanel, "find");

        jLabel9.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel9.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel9.setText("Session objects");

        lastObj_list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        lastObj_list.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lastObj_listMouseClicked(evt);
            }
        });

        jScrollPane1.setViewportView(lastObj_list);

        viewObj_button.setFont(new java.awt.Font("Verdana", 1, 10));
        viewObj_button.setText("View selected object");
        viewObj_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewObj_buttonActionPerformed(evt);
            }
        });

        jLabel5.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel5.setText("Session functions");

        lastFunc_list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane3.setViewportView(lastFunc_list);

        javax.swing.GroupLayout objsPanelLayout = new javax.swing.GroupLayout(objsPanel);
        objsPanel.setLayout(objsPanelLayout);
        objsPanelLayout.setHorizontalGroup(
            objsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(objsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(objsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(jLabel9, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(viewObj_button, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE))
                .addContainerGap())
        );
        objsPanelLayout.setVerticalGroup(
            objsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(objsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel9)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(viewObj_button)
                .addGap(20, 20, 20)
                .addComponent(jLabel5)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(115, Short.MAX_VALUE))
        );
        jPanel5.add(objsPanel, "objects");

        jPanel3.add(jPanel5, java.awt.BorderLayout.CENTER);

        showHelp_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/help.png")));
        showHelp_button.setToolTipText("Show PLaSM Library Reference");
        showHelp_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showHelp_buttonActionPerformed(evt);
            }
        });

        jPanel6.add(showHelp_button);

        defObj_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/defobj.png")));
        defObj_button.setToolTipText("Show last defined objects");
        defObj_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                defObj_buttonActionPerformed(evt);
            }
        });

        jPanel6.add(defObj_button);

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/find.png")));
        jButton2.setToolTipText("Find and replace");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jPanel6.add(jButton2);

        jPanel3.add(jPanel6, java.awt.BorderLayout.SOUTH);

        jSplitPane2.setRightComponent(jPanel3);

        jSplitPane1.setRightComponent(jSplitPane2);

        jToolBar2.setFloatable(false);
        caretLine_label.setFont(new java.awt.Font("Verdana", 1, 10));
        caretLine_label.setText(" - ");

        saving_label.setFont(new java.awt.Font("Verdana", 1, 10));
        saving_label.setForeground(new java.awt.Color(204, 204, 255));
        saving_label.setText("  Saving...");
        saving_label.setToolTipText("");

        jLabel1.setFont(new java.awt.Font("Verdana", 1, 10));
        jLabel1.setText("PLaSM Interpreter Status:  ");
        jLabel1.setPreferredSize(new java.awt.Dimension(160, 15));

        jLabel2.setBackground(java.awt.Color.red);
        jLabel2.setFont(new java.awt.Font("Bitstream Vera Sans Mono", 1, 10));
        jLabel2.setForeground(new java.awt.Color(255, 255, 255));
        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel2.setText("NOT RUNNING");
        jLabel2.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel2.setOpaque(true);
        jLabel2.setPreferredSize(new java.awt.Dimension(90, 15));

        sendingCommand_label.setFont(new java.awt.Font("Verdana", 1, 10));
        sendingCommand_label.setForeground(new java.awt.Color(204, 204, 255));
        sendingCommand_label.setText("   Sending command...");

        working_label.setFont(new java.awt.Font("Verdana", 1, 10));
        working_label.setForeground(new java.awt.Color(204, 204, 255));
        working_label.setText("   Working... ");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(working_label)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(sendingCommand_label)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(saving_label)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 416, Short.MAX_VALUE)
                .addComponent(caretLine_label)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(caretLine_label)
                    .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 14, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(working_label)
                    .addComponent(sendingCommand_label)
                    .addComponent(saving_label))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jToolBar2.add(jPanel2);

        jMenuBar1.setFont(new java.awt.Font("Bitstream Vera Sans", 0, 12));
        file_dropmenu.setMnemonic('F');
        file_dropmenu.setText("File");
        file_dropmenu.setFont(new java.awt.Font("Verdana", 0, 12));
        newFile_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        newFile_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        newFile_menu.setText("New");
        newFile_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newFile_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(newFile_menu);

        openFile_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        openFile_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        openFile_menu.setText("Load...");
        openFile_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openFile_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(openFile_menu);

        saveFile_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        saveFile_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        saveFile_menu.setText("Save");
        saveFile_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveFile_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(saveFile_menu);

        saveAs_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        saveAs_menu.setText("Save as...");
        saveAs_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAs_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(saveAs_menu);

        saveAll_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        saveAll_menu.setText("Save all");
        saveAll_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAll_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(saveAll_menu);

        file_dropmenu.add(jSeparator1);

        closeFile_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        closeFile_menu.setText("Close current");
        closeFile_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeFile_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(closeFile_menu);

        file_dropmenu.add(jSeparator2);

        quit_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        quit_menu.setText("Quit");
        quit_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quit_menuActionPerformed(evt);
            }
        });

        file_dropmenu.add(quit_menu);

        jMenuBar1.add(file_dropmenu);

        modify_dropmenu.setMnemonic('M');
        modify_dropmenu.setText("Modify");
        modify_dropmenu.setFont(new java.awt.Font("Verdana", 0, 12));
        undo_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Z, java.awt.event.InputEvent.CTRL_MASK));
        undo_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        undo_menu.setText("Undo");
        undo_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                undo_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(undo_menu);

        redo_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Y, java.awt.event.InputEvent.CTRL_MASK));
        redo_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        redo_menu.setText("Redo");
        redo_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                redo_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(redo_menu);

        modify_dropmenu.add(jSeparator4);

        cut_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.event.InputEvent.CTRL_MASK));
        cut_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        cut_menu.setText("Cut");
        cut_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cut_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(cut_menu);

        copy_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.CTRL_MASK));
        copy_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        copy_menu.setText("Copy");
        copy_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                copy_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(copy_menu);

        paste_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, java.awt.event.InputEvent.CTRL_MASK));
        paste_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        paste_menu.setText("Paste");
        paste_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                paste_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(paste_menu);

        modify_dropmenu.add(jSeparator5);

        tilde_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.CTRL_MASK));
        tilde_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        tilde_menu.setText("Insert tilde");
        tilde_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tilde_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(tilde_menu);

        modify_dropmenu.add(jSeparator9);

        autoclosebrackets_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        autoclosebrackets_menu.setText("Autoclose brackets");
        autoclosebrackets_menu.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                autoclosebrackets_menuItemStateChanged(evt);
            }
        });

        modify_dropmenu.add(autoclosebrackets_menu);

        modify_dropmenu.add(jSeparator10);

        incEditFont_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_PLUS, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.SHIFT_MASK));
        incEditFont_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        incEditFont_menu.setText("Increase editor font");
        incEditFont_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                incEditFont_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(incEditFont_menu);

        decEditFont_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_MINUS, java.awt.event.InputEvent.ALT_MASK | java.awt.event.InputEvent.SHIFT_MASK));
        decEditFont_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        decEditFont_menu.setText("Decrease editor font");
        decEditFont_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decEditFont_menuActionPerformed(evt);
            }
        });

        modify_dropmenu.add(decEditFont_menu);

        jMenuBar1.add(modify_dropmenu);

        plasm_dropmenu.setMnemonic('P');
        plasm_dropmenu.setText("PLaSM");
        plasm_dropmenu.setFont(new java.awt.Font("Verdana", 0, 12));
        startProcess_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        startProcess_menu.setText("Start process");
        startProcess_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startProcess_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(startProcess_menu);

        stopProcess_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        stopProcess_menu.setText("Stop process");
        stopProcess_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopProcess_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(stopProcess_menu);

        plasm_dropmenu.add(jSeparator3);

        clearConsole_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        clearConsole_menu.setText("Clear console");
        clearConsole_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearConsole_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(clearConsole_menu);

        incConsoleFont_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_PLUS, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        incConsoleFont_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        incConsoleFont_menu.setText("Increase console font");
        incConsoleFont_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                incConsoleFont_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(incConsoleFont_menu);

        decConsoleFont_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_MINUS, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        decConsoleFont_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        decConsoleFont_menu.setText("Decrease console font");
        decConsoleFont_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decConsoleFont_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(decConsoleFont_menu);

        plasm_dropmenu.add(jSeparator6);

        evalSelection_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        evalSelection_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        evalSelection_menu.setText("Evaluate selection");
        evalSelection_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalSelection_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(evalSelection_menu);

        evalFile_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B, java.awt.event.InputEvent.CTRL_MASK));
        evalFile_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        evalFile_menu.setText("Evaluate current file");
        evalFile_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalFile_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(evalFile_menu);

        view_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.CTRL_MASK));
        view_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        view_menu.setText("View selection (OpenGL)");
        view_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(view_menu);

        plasm_dropmenu.add(jSeparator7);

        evalSelScheme_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        evalSelScheme_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        evalSelScheme_menu.setText("Eval selection as Scheme");
        evalSelScheme_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalSelScheme_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(evalSelScheme_menu);

        evalFileScheme_menu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        evalFileScheme_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        evalFileScheme_menu.setText("Eval current file as Scheme");
        evalFileScheme_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evalFileScheme_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(evalFileScheme_menu);

        plasm_dropmenu.add(jSeparator8);

        brokenPipe_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        brokenPipe_menu.setText("Ignore broken pipe");
        brokenPipe_menu.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                brokenPipe_menuItemStateChanged(evt);
            }
        });

        plasm_dropmenu.add(brokenPipe_menu);

        configure_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        configure_menu.setText("Configure...");
        configure_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                configure_menuActionPerformed(evt);
            }
        });

        plasm_dropmenu.add(configure_menu);

        jMenuBar1.add(plasm_dropmenu);

        about_dropmenu.setMnemonic('A');
        about_dropmenu.setText("About");
        about_dropmenu.setFont(new java.awt.Font("Verdana", 0, 12));
        aboutPlasm_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        aboutPlasm_menu.setText("About PLaSM");
        aboutPlasm_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutPlasm_menuActionPerformed(evt);
            }
        });

        about_dropmenu.add(aboutPlasm_menu);

        aboutPlasmIde_menu.setFont(new java.awt.Font("Verdana", 0, 12));
        aboutPlasmIde_menu.setText("About PLaSM5 IDE");
        aboutPlasmIde_menu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutPlasmIde_menuActionPerformed(evt);
            }
        });

        about_dropmenu.add(aboutPlasmIde_menu);

        jMenuBar1.add(about_dropmenu);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 982, Short.MAX_VALUE)
            .addComponent(jToolBar2, javax.swing.GroupLayout.DEFAULT_SIZE, 982, Short.MAX_VALUE)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 982, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 38, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 728, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jToolBar2, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void lastObj_listMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lastObj_listMouseClicked
        if (evt.getClickCount() == 2) {          // Double-click
                // Get item index
            int index = this.lastObj_list.locationToIndex(evt.getPoint());
            this.lastObj_list.setSelectedIndex(index);
            String sel = this.lastObj_list.getSelectedValue().toString();
            this.plasm.evalPlasmCommand("VIEW:"+sel+";");
        }
    }//GEN-LAST:event_lastObj_listMouseClicked

    private void viewObj_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewObj_buttonActionPerformed
        if(!this.lastObj_list.isSelectionEmpty()) {
            String sel =  this.lastObj_list.getSelectedValue().toString();
            this.plasm.evalPlasmCommand("VIEW:"+sel+";");
        }
    }//GEN-LAST:event_viewObj_buttonActionPerformed

    private void jToggleButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton1ActionPerformed
        if(this.jToggleButton1.isSelected()) {
            this.jSplitPane2.setVisible(false);
            this.divLocation = this.jSplitPane1.getDividerLocation();
            this.jSplitPane1.setDividerLocation(1.0);
            this.jSplitPane1.invalidate();
        } else {
            this.jSplitPane2.setVisible(true);
            this.jSplitPane1.setDividerLocation(this.divLocation);
            this.jSplitPane1.invalidate();
        }
    }//GEN-LAST:event_jToggleButton1ActionPerformed

    private void replaceall_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_replaceall_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String toFind = this.search_text.getText();
            String toReplace = this.replace_text.getText();
            if(toFind != null && toFind.trim().length() > 0 && toReplace != null) {
                 replaceInTab(pane,toFind,toReplace);
            }
        }
    }//GEN-LAST:event_replaceall_buttonActionPerformed

    private void replace_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_replace_buttonActionPerformed
        // Algoritmo
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String toFind = this.search_text.getText();
            String toReplace = this.replace_text.getText();
            if(toFind != null && toFind.trim().length() > 0 && toReplace != null) {
                int index = this.currentFindPosition - toFind.length();
                try {
                    // Prendi this.currentFindPosition
                    // if(index > 0)
                    //    System.out.println("DEBUG: currentFindPosition: " + this.currentFindPosition + " - Word to find: " + toFind + " - Word catched: " + pane.getJSyntaxTextPane().getDocument().getText(index,toFind.length()));
                    if(index > 0 && pane.getJSyntaxTextPane().getDocument().getText(index,toFind.length()).equals(toFind)) {
                    // Se la parola che si trova tra this.currentFindPosition - toFind.length e this.currentFindPosition == toFind, sostituiscila
                        pane.getJSyntaxTextPane().getDocument().remove(index,toFind.length());
                        pane.getJSyntaxTextPane().getDocument().insertString(index,toReplace,new SimpleAttributeSet());
                        this.currentFindPosition = index + toReplace.length();
                    } else {
                    // Altrimenti chiama il find
                    //    System.out.println("Cerco altra parola");
                        findInCurrentTab(pane.getJSyntaxTextPane(),toFind);
                    //    System.out.println("Termine ricerca: " + this.currentFindPosition);
                    }
                } catch(BadLocationException ex) {
                    // System.out.println("BadLocationException: " + this.currentFindPosition + " - Length: " + toReplace.length());
                }
            }
        }
        
    }//GEN-LAST:event_replace_buttonActionPerformed

    private void next_button1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_next_button1ActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String toFind = this.search_text.getText();
            if(toFind != null && toFind.trim().length() > 0) {
                 findInCurrentTab(pane.getJSyntaxTextPane(),toFind);
            }
        }
    }//GEN-LAST:event_next_button1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        CardLayout cl = (CardLayout) this.jPanel5.getLayout();
        cl.show(this.jPanel5,"find");
    }//GEN-LAST:event_jButton2ActionPerformed

    private void defObj_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_defObj_buttonActionPerformed
        CardLayout cl = (CardLayout) this.jPanel5.getLayout();
        cl.show(this.jPanel5,"objects");
    }//GEN-LAST:event_defObj_buttonActionPerformed

    private void showHelp_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showHelp_buttonActionPerformed
        CardLayout cl = (CardLayout) this.jPanel5.getLayout();
        cl.show(this.jPanel5,"plasmref");
    }//GEN-LAST:event_showHelp_buttonActionPerformed

    private void decEditFont_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decEditFont_menuActionPerformed
        this.editorFontSize--;
        for(Component jc : this.jTabbedPane1.getComponents()) {
            PlasmPane pane = (PlasmPane) jc;
            pane.getJSyntaxTextPane().setFont(new Font("Dialog",0,this.editorFontSize));
        }
    }//GEN-LAST:event_decEditFont_menuActionPerformed

    private void incEditFont_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_incEditFont_menuActionPerformed
        this.editorFontSize++;
        for(Component jc : this.jTabbedPane1.getComponents()) {
            PlasmPane pane = (PlasmPane) jc;
            pane.getJSyntaxTextPane().setFont(new Font("Dialog",0,this.editorFontSize));
        }
    }//GEN-LAST:event_incEditFont_menuActionPerformed

    private void decConsoleFont_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decConsoleFont_menuActionPerformed
        this.consoleFontSize--;
        this.jTextArea1.setFont(this.jTextArea1.getFont().deriveFont(1,this.consoleFontSize));
    }//GEN-LAST:event_decConsoleFont_menuActionPerformed

    private void incConsoleFont_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_incConsoleFont_menuActionPerformed
        this.consoleFontSize++;
        this.jTextArea1.setFont(this.jTextArea1.getFont().deriveFont(1,this.consoleFontSize));
    }//GEN-LAST:event_incConsoleFont_menuActionPerformed

    
    private void jTabbedPane1StateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jTabbedPane1StateChanged
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String titl = pane.getData().getPath();
            if(titl != null)
                this.setTitlePath(titl);
            else
                this.setTitlePath(pane.getData().getFilename());
        } else {
            this.setTitlePath("");
        }
        this.currentFindPosition = 0;
    }//GEN-LAST:event_jTabbedPane1StateChanged

    private void brokenPipe_menuItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_brokenPipe_menuItemStateChanged
        this.plasm.setNoKillOnBrokenPipe(this.brokenPipe_menu.isSelected());
    }//GEN-LAST:event_brokenPipe_menuItemStateChanged

    private void saveAll_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAll_menuActionPerformed
        this.saving_label.setForeground(new java.awt.Color(204, 54, 35));
        for(Component jc : this.jTabbedPane1.getComponents()) {
            PlasmPane pane = (PlasmPane) jc;
            saveFileWithoutAsk(pane);
        }
        this.saving_label.setForeground(new java.awt.Color(204, 204, 255));
        
        this.jTabbedPane1StateChanged(null);
    }//GEN-LAST:event_saveAll_menuActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
       saveFile();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void saveAs_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAs_menuActionPerformed
        saveFile();
    }//GEN-LAST:event_saveAs_menuActionPerformed

    private void autoclosebrackets_menuItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_autoclosebrackets_menuItemStateChanged
        BracketOperationsListener.setAutocloseBracket( this.autoclosebrackets_menu.isSelected() );
    }//GEN-LAST:event_autoclosebrackets_menuItemStateChanged

    private void tilde_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tilde_menuActionPerformed
        /* Add tilde in the current panel/caret position */
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            this.ops.insertTextInCaret(pane.getJSyntaxTextPane(),"~");
        }
    }//GEN-LAST:event_tilde_menuActionPerformed

    private void view_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_view_menuActionPerformed
        viewOpenGL_buttonActionPerformed(evt);
    }//GEN-LAST:event_view_menuActionPerformed

    private void aboutPlasmIde_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutPlasmIde_menuActionPerformed
        JOptionPane.showMessageDialog(this,"JPLaSM 5 IDE\r\nDeveloped by Dario Lucia\r\n\r\nContacts: dario.lucia@gmail.com\r\n\r\nHomePage: http://www.capponcino.it/dario","JPlasm 5 IDE - Version 1.1.0.8",JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_aboutPlasmIde_menuActionPerformed

    private void aboutPlasm_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutPlasm_menuActionPerformed
        URL urlIcon = this.getClass().getResource("/ppp.png");
        log(urlIcon.toString());
        StringBuffer buff = new StringBuffer();
        buff.append("Programming LAnguage for Solid Modeling\r\nDeveloped by the CAD Group at the Universities of Rome \"La Sapienza\" and \"Roma Tre\"\r\n\r\nDevelopers:\r\n\r\n Alberto Paoluzzi\r\n Giorgio Scorzelli\r\n Franco Milicchio\r\n Simone Portuesi\r\n\r\n Project page: http://www.plasm.net");

        JOptionPane.showMessageDialog(this, buff.toString() ,"About PLaSM...",JOptionPane.INFORMATION_MESSAGE,new ImageIcon( urlIcon ));
    }//GEN-LAST:event_aboutPlasm_menuActionPerformed

    private void evalFileScheme_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalFileScheme_menuActionPerformed
        evalAllScheme_buttonActionPerformed(evt);
    }//GEN-LAST:event_evalFileScheme_menuActionPerformed

    private void evalSelScheme_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalSelScheme_menuActionPerformed
        evalSelScheme_buttonActionPerformed(evt);
    }//GEN-LAST:event_evalSelScheme_menuActionPerformed

    private void evalAllScheme_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalAllScheme_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String cmd = pane.getJSyntaxTextPane().getText();
            if(cmd != null && cmd.trim().length() > 0)
                this.plasm.evalSchemeCommand(cmd);
        }
    }//GEN-LAST:event_evalAllScheme_buttonActionPerformed

    private void evalSelScheme_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalSelScheme_buttonActionPerformed
            PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String cmd = pane.getJSyntaxTextPane().getSelectedText();
            if(cmd != null && cmd.trim().length() > 0)
                this.plasm.evalSchemeCommand(cmd);
        }
    }//GEN-LAST:event_evalSelScheme_buttonActionPerformed

    private void viewOpenGL_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewOpenGL_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String cmd = pane.getJSyntaxTextPane().getSelectedText();
            if(cmd != null && cmd.trim().length() > 0)
                this.plasm.evalPlasmCommand("VIEW:" + cmd + ";");
        }
    }//GEN-LAST:event_viewOpenGL_buttonActionPerformed

    private void evalFile_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalFile_menuActionPerformed
        evalAll_buttonActionPerformed(evt);
    }//GEN-LAST:event_evalFile_menuActionPerformed

    private void evalSelection_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalSelection_menuActionPerformed
        evalSel_buttonActionPerformed(evt);
    }//GEN-LAST:event_evalSelection_menuActionPerformed

    private void evalAll_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalAll_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String cmd = pane.getJSyntaxTextPane().getText();
            if(cmd != null && cmd.trim().length() > 0)
                this.plasm.evalPlasmCommand(cmd);
        }
    }//GEN-LAST:event_evalAll_buttonActionPerformed

    private void evalSel_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evalSel_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            String cmd = pane.getJSyntaxTextPane().getSelectedText();
            if(cmd != null && cmd.trim().length() > 0)
                this.plasm.evalPlasmCommand(cmd);
        }
    }//GEN-LAST:event_evalSel_buttonActionPerformed

    private void stopProcess_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopProcess_menuActionPerformed
        stopPlasm_buttonActionPerformed(evt);
    }//GEN-LAST:event_stopProcess_menuActionPerformed

    private void startProcess_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startProcess_menuActionPerformed
        startPlasm_buttonActionPerformed(evt);
    }//GEN-LAST:event_startProcess_menuActionPerformed

    private void stopPlasm_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopPlasm_buttonActionPerformed
        this.plasm.stopPlasm();
    }//GEN-LAST:event_stopPlasm_buttonActionPerformed

    private void startPlasm_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startPlasm_buttonActionPerformed
        this.plasm.startPlasm(this.configForm.getPlasm(),this.configForm.getMZScheme());
    }//GEN-LAST:event_startPlasm_buttonActionPerformed

    private void clearConsole_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearConsole_menuActionPerformed
        this.jTextArea1.setText("");
    }//GEN-LAST:event_clearConsole_menuActionPerformed

    private void configure_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_configure_menuActionPerformed
        this.configForm.setVisible(true);
    }//GEN-LAST:event_configure_menuActionPerformed

    private void saveFile_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveFile_menuActionPerformed
        saveFileWithoutAsk();
    }//GEN-LAST:event_saveFile_menuActionPerformed

    private void openFile_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openFile_menuActionPerformed
        openFile();
    }//GEN-LAST:event_openFile_menuActionPerformed

    private void redo_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_redo_menuActionPerformed
        redo_buttonActionPerformed(evt);
    }//GEN-LAST:event_redo_menuActionPerformed

    private void undo_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_undo_menuActionPerformed
         undo_buttonActionPerformed(evt);
    }//GEN-LAST:event_undo_menuActionPerformed

    private void redo_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_redo_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null)
            pane.performRedo();
    }//GEN-LAST:event_redo_buttonActionPerformed

    private void undo_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_undo_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null)
            pane.performUndo();
    }//GEN-LAST:event_undo_buttonActionPerformed

    private void paste_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_paste_menuActionPerformed
        paste_buttonActionPerformed(evt);
    }//GEN-LAST:event_paste_menuActionPerformed

    private void copy_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_copy_menuActionPerformed
        copy_buttonActionPerformed(evt);
    }//GEN-LAST:event_copy_menuActionPerformed

    private void cut_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cut_menuActionPerformed
        cut_buttonActionPerformed(evt);
    }//GEN-LAST:event_cut_menuActionPerformed

    private void copy_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_copy_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            pane.performCopy(evt);
        }
    }//GEN-LAST:event_copy_buttonActionPerformed

    private void paste_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_paste_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
         if(pane != null) {
            pane.performPaste(evt);
         }
    }//GEN-LAST:event_paste_buttonActionPerformed

    private void cut_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cut_buttonActionPerformed
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
         if(pane != null) {
            pane.performCut(evt);
         }
    }//GEN-LAST:event_cut_buttonActionPerformed

    private void windowClosingHandler(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_windowClosingHandler
        quit();
    }//GEN-LAST:event_windowClosingHandler

    private void quit_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_quit_menuActionPerformed
        quit();
    }//GEN-LAST:event_quit_menuActionPerformed

    private void saveFile_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveFile_buttonActionPerformed
        saveFileWithoutAsk();
    }//GEN-LAST:event_saveFile_buttonActionPerformed

    private void openFile_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openFile_buttonActionPerformed
        openFile();
    }//GEN-LAST:event_openFile_buttonActionPerformed

    private void closeFile_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeFile_menuActionPerformed
        closeCurrent();
    }//GEN-LAST:event_closeFile_menuActionPerformed

    private void newFile_menuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newFile_menuActionPerformed
        newFile();
    }//GEN-LAST:event_newFile_menuActionPerformed

    private void closeFile_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeFile_buttonActionPerformed
        closeCurrent();
    }//GEN-LAST:event_closeFile_buttonActionPerformed

    private void newFile_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newFile_buttonActionPerformed
        newFile();
    }//GEN-LAST:event_newFile_buttonActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        final String[] argss = args;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                Main m = new Main();
                m.setParameters(argss);
                m.setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutPlasmIde_menu;
    private javax.swing.JMenuItem aboutPlasm_menu;
    private javax.swing.JMenu about_dropmenu;
    private javax.swing.JCheckBoxMenuItem autoclosebrackets_menu;
    private javax.swing.JCheckBoxMenuItem brokenPipe_menu;
    private javax.swing.JLabel caretLine_label;
    private javax.swing.JMenuItem clearConsole_menu;
    private javax.swing.JButton closeFile_button;
    private javax.swing.JMenuItem closeFile_menu;
    private javax.swing.JMenuItem configure_menu;
    private javax.swing.JScrollPane consolePanel;
    private javax.swing.JButton copy_button;
    private javax.swing.JMenuItem copy_menu;
    private javax.swing.JButton cut_button;
    private javax.swing.JMenuItem cut_menu;
    private javax.swing.JMenuItem decConsoleFont_menu;
    private javax.swing.JMenuItem decEditFont_menu;
    private javax.swing.JButton defObj_button;
    private javax.swing.JButton evalAllScheme_button;
    private javax.swing.JButton evalAll_button;
    private javax.swing.JMenuItem evalFileScheme_menu;
    private javax.swing.JMenuItem evalFile_menu;
    private javax.swing.JButton evalSelScheme_button;
    private javax.swing.JMenuItem evalSelScheme_menu;
    private javax.swing.JButton evalSel_button;
    private javax.swing.JMenuItem evalSelection_menu;
    private javax.swing.JMenu file_dropmenu;
    private javax.swing.JPanel findPanel;
    private javax.swing.JMenuItem incConsoleFont_menu;
    private javax.swing.JMenuItem incEditFont_menu;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JEditorPane jEditorPane1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator10;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JSeparator jSeparator9;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JSplitPane jSplitPane2;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JToggleButton jToggleButton1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JToolBar jToolBar2;
    private javax.swing.JList lastFunc_list;
    private javax.swing.JList lastObj_list;
    private javax.swing.JMenu modify_dropmenu;
    private javax.swing.JButton newFile_button;
    private javax.swing.JMenuItem newFile_menu;
    private javax.swing.JButton next_button1;
    private javax.swing.JPanel objsPanel;
    private javax.swing.JButton openFile_button;
    private javax.swing.JMenuItem openFile_menu;
    private javax.swing.JButton paste_button;
    private javax.swing.JMenuItem paste_menu;
    private javax.swing.JMenu plasm_dropmenu;
    private javax.swing.JMenuItem quit_menu;
    private javax.swing.JButton redo_button;
    private javax.swing.JMenuItem redo_menu;
    private javax.swing.JPanel referencePanel;
    private javax.swing.JButton replace_button;
    private javax.swing.JTextField replace_text;
    private javax.swing.JButton replaceall_button;
    private javax.swing.JMenuItem saveAll_menu;
    private javax.swing.JMenuItem saveAs_menu;
    private javax.swing.JButton saveFile_button;
    private javax.swing.JMenuItem saveFile_menu;
    private javax.swing.JLabel saving_label;
    private javax.swing.JTextField search_text;
    private javax.swing.JLabel sendingCommand_label;
    private javax.swing.JButton showHelp_button;
    private javax.swing.JButton startPlasm_button;
    private javax.swing.JMenuItem startProcess_menu;
    private javax.swing.JButton stopPlasm_button;
    private javax.swing.JMenuItem stopProcess_menu;
    private javax.swing.JMenuItem tilde_menu;
    private javax.swing.JButton undo_button;
    private javax.swing.JMenuItem undo_menu;
    private javax.swing.JButton viewObj_button;
    private javax.swing.JButton viewOpenGL_button;
    private javax.swing.JMenuItem view_menu;
    private javax.swing.JLabel working_label;
    // End of variables declaration//GEN-END:variables
     
    /* Support methods */
    private void appendTab(PlasmPane panel) {
        /* Attach right click listener */
        final JTextPane jta = panel.getJSyntaxTextPane();
        /* Set Font */
        jta.setFont(new Font("Dialog",0,this.editorFontSize));
        /* For popup menu */
        jta.addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent mouseEvent) {
                if(mouseEvent.getButton() == MouseEvent.BUTTON3) {
                    JPopupMenu jpm = getTabMenu();
                    /* Reset some enabled label (i.e. Eval and View) */
                    String selText = jta.getSelectedText();
                    if(selText == null || selText.equals("")) {
                        jpm.getSubElements()[3].getComponent().setEnabled(false);
                        jpm.getSubElements()[4].getComponent().setEnabled(false);
                    } else {
                        jpm.getSubElements()[3].getComponent().setEnabled(true);
                        jpm.getSubElements()[4].getComponent().setEnabled(true);
                    }
                    jpm.show(mouseEvent.getComponent(),mouseEvent.getX(),mouseEvent.getY());
                }
            }
            public void mouseEntered(MouseEvent mouseEvent) {
            }
            public void mouseExited(MouseEvent mouseEvent) {
            }
            public void mousePressed(MouseEvent mouseEvent) {
            }
            public void mouseReleased(MouseEvent mouseEvent) {
            }
        });
        
        /* For line number: watch impact on performance ! */
        jta.addCaretListener(this);
        
        this.jTabbedPane1.addTab(panel.getData().getFilename(),panel);
        this.jTabbedPane1.setSelectedComponent(panel);
        panel.getJSyntaxTextPane().requestFocusInWindow();
        this.jTabbedPane1.validate();
    }
    
    private void closeTab(int sel) {
        
        if(sel>=0) {
            PlasmPane selPanel = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
            if(selPanel.getData().isModified()) {
                /* if modified, ask if close or cancel action */       
                int answer = JOptionPane.showOptionDialog(this,"File " + selPanel.getData().getFilename() + " is not saved. You will lost your changes. Do you want to close the file?","Closing a not saved file",JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE,null,null,null);
                if(answer == 0) {
                    this.jTabbedPane1.remove(sel);
                }
            } else {
                this.jTabbedPane1.remove(sel);
            }
            this.caretLine_label.setText("-");
        }
        
    }
   
    /* JTextArea Tab Menu */
   private JPopupMenu getTabMenu() {
       if(tabMenu == null) {
           /* Init only once */
           
           tabMenu = new JPopupMenu();
           JMenuItem menuItem = new JMenuItem("Cut");
           menuItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent actionEvent) {
                   cut_buttonActionPerformed(actionEvent);
               }
           });
           menuItem.setFont(new java.awt.Font("Verdana", 0, 12));
           tabMenu.add(menuItem);
           
           menuItem = new JMenuItem("Copy");
           menuItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent actionEvent) {
                   copy_buttonActionPerformed(actionEvent);
               }
           });
           menuItem.setFont(new java.awt.Font("Verdana", 0, 12));
           tabMenu.add(menuItem);
           
           menuItem = new JMenuItem("Paste");
           menuItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent actionEvent) {
                   paste_buttonActionPerformed(actionEvent);
               }
           });
           menuItem.setFont(new java.awt.Font("Verdana", 0, 12));
           tabMenu.add(menuItem);
           
           tabMenu.addSeparator();
           
           menuItem = new JMenuItem("Eval selection");
           menuItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent actionEvent) {
                   evalSelection_menuActionPerformed(actionEvent);
               }
           });
           menuItem.setFont(new java.awt.Font("Verdana", 0, 12));
           tabMenu.add(menuItem);
           
           menuItem = new JMenuItem("View selection");
           menuItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent actionEvent) {
                   viewOpenGL_buttonActionPerformed(actionEvent);
               }
           });
           menuItem.setFont(new java.awt.Font("Verdana", 0, 12));
           tabMenu.add(menuItem);
           
       }
       return tabMenu;
   }
    
    /* Action  methods */
    protected void newFile() {
        PlasmPane panel = this.ops.createNewTab();
        appendTab(panel);
    }
    
    protected void closeCurrent() {
        int sel = this.jTabbedPane1.getSelectedIndex();
        closeTab(sel);
    }
    
    protected void openFile() {
        this.openDialog.setMultiSelectionEnabled(true);
        int res = this.openDialog.showOpenDialog(this);
        if(res == JFileChooser.APPROVE_OPTION) {
            File[] fs = this.openDialog.getSelectedFiles();
            
            for(File f : fs) {
                /* Crea tab */
                PlasmPane panel = this.ops.createNewTab();
                panel.getData().setPath(f.getPath());
                panel.getData().setFilename(f.getName());
                try {   
                    panel.getJSyntaxTextPane().setText(this.ops.loadFile(f));
                    /* aggiungi tab */
                    appendTab(panel);
                    /* reset status */
                    panel.getData().setModified(false);

                } catch (FileNotFoundException ex) {
                    ex.printStackTrace();
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this,"File not found: " + f.getPath(),"Error...",JOptionPane.ERROR_MESSAGE);
                } catch (IOException ex) {
                    ex.printStackTrace();
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this,"IOException while opening file " + f.getPath(),"Error...",JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }
    
    /* Save the current file */
    protected void saveFile() {
        this.saving_label.setForeground(new java.awt.Color(204, 54, 35));
        /* Catch current tab */
        PlasmPane selPanel = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        saveFile(selPanel);
        this.saving_label.setForeground(new java.awt.Color(204, 204, 255));
 
        this.jTabbedPane1StateChanged(null);
    }
    
    /* Save without asking if the file is already on disk (read: if its path is != null) */
    private void saveFileWithoutAsk() {
        this.saving_label.setForeground(new java.awt.Color(204, 54, 35));
        PlasmPane selPanel = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        saveFileWithoutAsk(selPanel);
        this.saving_label.setForeground(new java.awt.Color(204, 204, 255));
        
        this.jTabbedPane1StateChanged(null);
    }
    
    private void saveFileWithoutAsk(PlasmPane selPanel) {
        if(selPanel != null) {
            if(selPanel.getData().getPath() != null) { 
                File f = new File(selPanel.getData().getPath());
                try {
                    this.ops.saveFile(f,selPanel);
                    resetTabChanges(selPanel);
                } catch (Exception e) {
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this,"Exception while saving file " + f.getPath() + ": " + e.getMessage(),"Error...",JOptionPane.ERROR_MESSAGE);
                }                
            } else {
                /* Ask */
                saveFile(selPanel);
            } 
        }
    }
    
    /* Save the given PlasmPane */
    private void saveFile(PlasmPane selPanel) {
        if(selPanel != null) {
            this.saveDialog.setMultiSelectionEnabled(false);
            
            if(selPanel.getData().getPath() != null)
                this.saveDialog.setSelectedFile(new File(selPanel.getData().getPath()));
            else
                 this.saveDialog.setSelectedFile(new File(selPanel.getData().getFilename()));
            
            int res = this.saveDialog.showSaveDialog(this);
            File f = saveDialog.getSelectedFile();
            if(res == JFileChooser.APPROVE_OPTION && f != null) {
                try {
                    this.ops.saveFile(f,selPanel);
                    resetTabChanges(selPanel);
                } catch (Exception e) {
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this,"Exception while saving file " + f.getPath() + ": " + e.getMessage(),"Error...",JOptionPane.ERROR_MESSAGE);
                    
                }
            }
        }
    }
    
    /* Ask PlasmPane to reset its title */
    private void resetTabChanges(PlasmPane selPanel) {
        /* Check title */
       int indOfC = this.jTabbedPane1.indexOfComponent(selPanel);
       if(indOfC >= 0) {
            if(selPanel.getData().isModified()) {
                this.jTabbedPane1.setTitleAt(indOfC,selPanel.getData().getFilename() + " (*)");
            } else {
                this.jTabbedPane1.setTitleAt(indOfC,selPanel.getData().getFilename());
            }
       
       }    
    }
    
    /* Quit the IDE */
    protected void quit() {
        /* before quitting, check if there is a file not saved and warn about that */
        boolean fnsFound = false;
      
        for(int i = 0;i<this.jTabbedPane1.getComponentCount() && !fnsFound;i++) {
            PlasmPane pane = (PlasmPane) this.jTabbedPane1.getComponentAt(i);
            if(pane.getData().isModified())
                fnsFound = true;
        }
        
        /* if fns was found, ask what to do, else quit */
        if(fnsFound) {
             int answer = JOptionPane.showOptionDialog(this,"Some files are not saved. You will lost your changes. Do you want to quit anyway?","Exiting without saving files",JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE,null,null,null);
             if(answer == 0) {
                    log("Exiting without saving");
                    this.plasm.stopPlasm();
                try {
                    Thread.sleep(100);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
                    /* Stop backup thread */
                    this.backup.interrupt();
                    System.exit(0);
             }
            
        } else {
            int answer = JOptionPane.showOptionDialog(this,"Are you sure you want to quit?","Quit Plasm IDE",JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE,null,null,null);
             if(answer == 0) {
            
                this.plasm.stopPlasm();
                try {
                    Thread.sleep(100);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
                
                /*  Stop backup thread */
                 this.backup.interrupt();
                 
                System.exit(0);
             }
        }
            
    }
    
    public void plasmStartedEvent() {
        this.jLabel2.setText("  RUNNING  ");
        this.jLabel2.setBackground(Color.GREEN);
        this.jLabel2.setForeground(Color.BLACK);
    }

    public void plasmStoppedEvent() {
        this.jLabel2.setText("NOT RUNNING");
        this.jLabel2.setBackground(Color.RED);
        this.jLabel2.setForeground(Color.WHITE);
        sendingCommand_label.setForeground(new java.awt.Color(204, 204, 255));
        working_label.setForeground(new java.awt.Color(204, 204, 255));
        resetListObjects();
    }

    public void plasmErrorEvent(String error) {
        writeToConsole("### ERROR SIGNALED ###");
        writeToConsole(error);
        
    }

    public void plasmCatchedOutputEvent(String output) {
        working_label.setForeground(new java.awt.Color(204, 54, 35));
        writeToConsole(output);
    }

    public void plasmRequestInputEvent(String input) {
        
        if(input.equals(PlasmWrapper.COMMAND_IN))
        {        
             sendingCommand_label.setForeground(new java.awt.Color(204, 54, 35));
        }
        else if(input.equals(PlasmWrapper.COMMAND_SENT)) 
        {          
             sendingCommand_label.setForeground(new java.awt.Color(204, 204, 255));
             working_label.setForeground(new java.awt.Color(204, 54, 35));
        }
        else
        {  
            // The command just sent .. Parse for DEF objects
            // Rapid parsing
            
            String lowInput = input.toLowerCase();
            int currPos = 0;
            int tempPos = 0;
            while((tempPos = lowInput.indexOf("def ",currPos)) != -1)  {
                currPos = tempPos + 4;
                // Skip se currPos si trova in un commento: basta contare i caratteri %
                // e vedere se son dispari
                if(!isInPlasmComments(lowInput,currPos)) {
                    int position = lowInput.indexOf("=",currPos);
                    if(position != -1) {
                        String sub = lowInput.substring(currPos,position);
                        if(sub.indexOf('(') == -1)
                            this.expObj.add(sub.trim());
                        else 
                            this.expFunc.add(sub.trim());
                    }
                }
            }
            redrawExportedLists();
            
            
        }
    } 
    // Handfull method for System.out.println
    private void log(String t) {
        System.out.println(t);
    }

    private void writeToConsole(String msg) {
        // this.jTextArea1.append(msg + NEW_LINE);
        this.jTextArea1.append(msg);
        this.jTextArea1.setCaretPosition(this.jTextArea1.getText().length()-1);
        // Check if end of the work...
        if (msg.trim().endsWith(">")) this.working_label.setForeground(new java.awt.Color(204, 204, 255));
    }

    private void findInCurrentTab(JTextPane jTextArea, String toFind) {
        String textToSearch = "";
        try {
            textToSearch = jTextArea.getDocument().getText(this.currentFindPosition,jTextArea.getDocument().getLength() - this.currentFindPosition);
        } catch (BadLocationException ex) {
            this.currentFindPosition = 0;
            findInCurrentTab(jTextArea,toFind);
            return;
        }
        int indexOf = textToSearch.indexOf(toFind);
        if(indexOf >= 0) {
            /* Evidenzia testo */
            jTextArea.requestFocusInWindow();
            jTextArea.setCaretPosition(indexOf);
            jTextArea.setSelectionStart(this.currentFindPosition + indexOf);
            jTextArea.setSelectionEnd(this.currentFindPosition + indexOf + toFind.length()) ;
            // this.jTextField2.requestFocusInWindow();
            /* Inc pointer for next search */
            this.currentFindPosition = this.currentFindPosition + indexOf + toFind.length() ;
        } else {
            // Reached the end: restart iff there is at least one occurrence in document, else end
            this.currentFindPosition = 0;
            try {
                if(jTextArea.getDocument().getText(0,jTextArea.getDocument().getLength()).indexOf(toFind) != -1) {
                    findInCurrentTab(jTextArea,toFind);
                    return;
                } else {
                    JOptionPane.showConfirmDialog(this,"No occurrences of the word " + toFind + " in selected tab","Information",JOptionPane.DEFAULT_OPTION,JOptionPane.INFORMATION_MESSAGE);
                }
            } catch (BadLocationException ex) {
                //
            }
        }
    }

    private void setTitlePath(String string) {
        this.setTitle("JPLaSM 5 IDE [" + string + "]");
    }

    public void caretUpdate(CaretEvent caretEvent) {
        PlasmPane pane = (PlasmPane) this.jTabbedPane1.getSelectedComponent();
        if(pane != null) {
            try {
                int nline = countLines(pane.getJSyntaxTextPane().getDocument().getText(0,caretEvent.getDot()))+1;
                this.caretLine_label.setText(String.valueOf(nline));
            } catch(BadLocationException e) {
                this.caretLine_label.setText("-");
            }
            
        }
    }

    private int countLines(String string) {
        int lines = 0;
        int bIndex = 0;
        int cIndex;
        while((cIndex = string.indexOf("\n",bIndex)) != -1) {
            ++lines;
            bIndex = cIndex + 1;
        }
        return lines;
    }

    private void setParameters(String[] args) {
        for(String s : args) {
                File f = new File(s);
                /* Crea tab */
                PlasmPane panel = this.ops.createNewTab();
                panel.getData().setPath(f.getPath());
                panel.getData().setFilename(f.getName());
                try {   
                    panel.getJSyntaxTextPane().setText(this.ops.loadFile(f));
                    /* aggiungi tab */
                    appendTab(panel);
                    /* reset status */
                    panel.getData().setModified(false);

                } catch (FileNotFoundException ex) {
                    ex.printStackTrace();
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this,"File not found: " + f.getPath(),"Error...",JOptionPane.ERROR_MESSAGE);
                } catch (IOException ex) {
                    ex.printStackTrace();
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this,"IOException while opening file " + f.getPath(),"Error...",JOptionPane.ERROR_MESSAGE);
                }
        }
    }

    private void replaceInTab(PlasmPane pane, String toFind, String toReplace) {
        Document doc = pane.getJSyntaxTextPane().getDocument();
        
        try {
            String text = doc.getText(0,doc.getLength());
            int index;
            int ncount = 0;
            
            while((index = text.indexOf(toFind)) != -1) {
                try {
                
                doc.remove(index,toFind.length());
                
                doc.insertString(index,toReplace,new SimpleAttributeSet());
                
                text = text.replaceFirst(toFind,toReplace);
                ++ncount;
                } catch(BadLocationException ex) {
                    // Simply skip
                }
            }
            JOptionPane.showConfirmDialog(this,ncount + " occurrences replaced","Information",JOptionPane.DEFAULT_OPTION,JOptionPane.INFORMATION_MESSAGE);
        } catch(BadLocationException e) {
            JOptionPane.showMessageDialog(this,"Unspecified error: replacement did not start","Error...",JOptionPane.ERROR_MESSAGE);
        }
    }

    private void redrawExportedLists() {       
        DefaultListModel objs = new DefaultListModel();
        DefaultListModel funcs = new DefaultListModel();
        for(String s : this.expObj)
            objs.addElement(s);
        for(String s : this.expFunc)
            funcs.addElement(s);
        this.lastObj_list.setModel(objs);
        this.lastFunc_list.setModel(funcs);
        this.lastFunc_list.invalidate();
        this.lastObj_list.invalidate();
    }

    private void resetListObjects() {
        this.expFunc.clear();
        this.expObj.clear();
        redrawExportedLists();
    }

    private boolean isInPlasmComments(String lowInput, int currPos) {
        String substr = lowInput.substring(0,currPos);
        int currentPos = 0;
        int found = 0;
        while((currentPos = substr.indexOf('%',currentPos)) != -1) {
            ++found;
            ++currentPos;
        }
        return ((found % 2) == 1);
    }
}
