/*
 * BracketOperationsListener.java
 *
 * Created on 6 ottobre 2007, 10.16
 *
 */

package it.jplasmide.gui.listeners;

import it.jplasmide.gui.utils.PlasmPane;
import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.JTextPane;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;

/**
 *
 * @author Dario Lucia
 */
public class BracketOperationsListener implements CaretListener,KeyListener{

    
    private static boolean autocloseBracket = false;
    
    public static void setAutocloseBracket(boolean t) {
        autocloseBracket = t;
    }
    
    private PlasmPane pane;
    
    /** Creates a new instance of BracketOperationsListener */
    public BracketOperationsListener(PlasmPane pane) {
        this.pane = pane;
    }

    public void caretUpdate(CaretEvent caretEvent) {
        findForBrackets();
    }

  
    public void keyTyped(KeyEvent keyEvent) {
            if(pane != null && autocloseBracket) {
                JTextPane txa = pane.getJSyntaxTextPane();
                char c = keyEvent.getKeyChar();
                switch(c) {
                     case '{':
                        insertTextInCaret(txa,"}");
                        break;
                     case '[':
                         insertTextInCaret(txa,"]");
                        break;
                     case '(':
                         insertTextInCaret(txa,")");
                        break;
                     case '<':
                         insertTextInCaret(txa,">");
                        break;
                }
            }
    }
  
    public void keyPressed(KeyEvent keyEvent) {
    }

    public void keyReleased(KeyEvent keyEvent) {
    }
    
 
    protected void insertTextInCaret(JTextPane txa, String t) {
        try {
           int oldCaret = txa.getCaretPosition();
           txa.getDocument().insertString(oldCaret,t,null);
           txa.setCaretPosition(oldCaret);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    
    private void removeBracketHighlighter(JTextPane txa) {
        Highlighter hilite = txa.getHighlighter();
        Highlighter.Highlight[] hilites = hilite.getHighlights();
    
        for (int i=0; i<hilites.length; i++) {
            if (hilites[i].getPainter() instanceof MyBracketPainter) {
                hilite.removeHighlight(hilites[i]);
            }
        }
    }
    
    private void addHighlighter(JTextPane txa , int position) {
        try {
            Highlighter hilite = txa.getHighlighter();
            hilite.addHighlight(position, position+1, bracketPainter);           
        } catch (BadLocationException e) {
        }
    }
    
    Highlighter.HighlightPainter bracketPainter = new MyBracketPainter(Color.PINK);
    // A private subclass of the default highlight painter
    class MyBracketPainter extends DefaultHighlighter.DefaultHighlightPainter {
        public MyBracketPainter(Color color) {
            super(color);
        }
    }
    
    /* Check if caret is after an open or closed bracket and check the other enclosing bracket */
    protected void findForBrackets() {
            JTextPane txa = pane.getJSyntaxTextPane();
            
            /* Caret update, so remove highlighter */
            removeBracketHighlighter(txa);
            
            int caretPos = txa.getCaretPosition();
            String testo = txa.getText();
       
            if(testo != null && testo.trim().length() > 0 && caretPos > 0) {
                char after = testo.charAt(caretPos-1);
                
                switch(after) {
                    case '<':
                        findBracket(txa,'>','<',true,caretPos);
                        break;
                    case '>':
                        findBracket(txa,'<','>',false,caretPos);
                        break;

                    case '(' :
                        findBracket(txa,')','(',true,caretPos);
                        break;

                    case ')' :
                        findBracket(txa,'(',')',false,caretPos);
                        break;

                    case '[' :
                        findBracket(txa,']','[',true,caretPos);
                        break;

                    case ']' :
                        findBracket(txa,'[',']',false,caretPos);
                        break;
                        
                    case '{' :
                        findBracket(txa,'}','{',true,caretPos);
                        break;

                    case '}' :
                        findBracket(txa,'{','}',false,caretPos);
                        break;
                }
            }
    }
    
    private void findBracket(JTextPane current, char toFound, char toCount, boolean forward, int currentCaretPos) {
        String text = current.getText();
        /* Move forward */
        if(forward) {
            int found = 0;
            for(int i = currentCaretPos;i<text.length();i++) {
                char c = text.charAt(i);
                if(c == toCount)
                    found++;
                else if(c == toFound) {
                    if(found == 0) {
                        addHighlighter(current,i);// tana!! fai highlight
                        break;
                    } else {
                        found--;
                    }
                }
            }
        } else {
        /* Move backward */
            int found = 0;
            for(int i = currentCaretPos-2 ;i>-1; i--) {
                char c = text.charAt(i);
                if(c==toCount)
                    found++;
                else if(c == toFound) {
                    if(found == 0) {
                          addHighlighter(current,i);// tana!! fai highlight
                          break;
                    } else {
                        found--;
                    }
                }
            }
        }
        
    }
    
    
}
