/*
 * PlasmGuiOperations.java
 *
 * Created on 8 ottobre 2007, 19.41
 *
 */

package it.jplasmide.gui.utils;

import it.jplasmide.doctool.DataBean;
import it.jplasmide.doctool.DataBeanContainer;
import it.jplasmide.gui.listeners.BracketOperationsListener;
import it.jplasmide.gui.listeners.NotifyDataTextChangeListener;
import java.awt.Color;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import jsyntax.*;

/**
 *
 * @author Dario Lucia
 */
public class PlasmGuiOperations {
    
    
    /* Autoinc variable for fresh -Unnamed- document */
    private int autoincFilename = 0;
     
    
    private JTabbedPane pane;

    private String NEW_LINE = System.getProperty("line.separator");
    private JSyntaxManager syntaxManager;
    
    /** Creates a new instance of PlasmGuiOperations */
    public PlasmGuiOperations(JTabbedPane pane, DataBeanContainer dbe) {
        this.pane = pane;
        
        /* Init syntax keywords */
        List<String> keys = new LinkedList<String>();
        List<DataBean> beans = dbe.getBeans("");
        
        // Only alhanumeric strings allowed
        for(DataBean bb : beans)
            if(bb.getName().matches("\\p{Alnum}+"))
                keys.add(bb.getName());
        
        keys.add("def");
        keys.add("end");
        keys.add("where");
        
        // By default, normal text is black, numbers are magenta
        JSyntaxKeywordManager keywords = new JSyntaxKeywordManager();
        keywords.addFormat(keys,Color.BLUE,false,false,false);
        
        keywords.addFormat("view",Color.RED,true,false,false);
        
        /* Init token divisors - Default tokens */
        JSyntaxTokenDivisorManager divisors = new JSyntaxTokenDivisorManager();
        
        this.syntaxManager = new JSyntaxManager(divisors,keywords);
        
    }
    
    public void insertTextInCaret(JTextPane txa, String t) {
        try {
           String text = txa.getText();
           int oldCaret = txa.getCaretPosition();
           txa.getDocument().insertString(oldCaret,t,null);
           if((oldCaret + t.length()) <= txa.getText().length() )
                txa.setCaretPosition(oldCaret+t.length());
           else
                txa.setCaretPosition(oldCaret);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public PlasmPane createNewTab() {
        final JSyntaxTextPane jta = this.syntaxManager.getJSyntaxTextPane(true);   
        // jta.setSize(this.jTabbedPane1.getWidth(),this.jTabbedPane1.getHeight());
        jta.setLocation(0,0);
     
        /* For tracking changes on text   */
        jta.getDocument().addDocumentListener(new NotifyDataTextChangeListener(jta,this.pane));
   
        /* Create plasm pane and automagically init undo/redo system */
        final PlasmPane panel = new PlasmPane(jta);   
        
        /* Set a qualified name */
        panel.getData().setFilename("Unnamed" + autoincFilename++ + ".psm");
        
        /* For bracket autoclose and highlighting */
        BracketOperationsListener bracketListener = new BracketOperationsListener(panel);
        
        jta.addCaretListener(bracketListener);
        jta.addKeyListener(bracketListener);
        
        
        return panel;
    }
    
   public void saveFile (File f, PlasmPane selPanel) throws IOException {
       /* Create file */
        if(!f.exists())
            f.createNewFile();

        /* Write tab to file */
        FileWriter fw = new FileWriter(f);
        BufferedWriter bw = new BufferedWriter(fw);
        bw.write(selPanel.getJSyntaxTextPane().getText());
        bw.close();
        /* Annotate changes into Data object */
        selPanel.getData().setFilename(f.getName());
        selPanel.getData().setPath(f.getPath());
        selPanel.getData().setModified(false);
   }
   
   public String loadFile (File f) throws FileNotFoundException, IOException {
        FileReader reader;
        reader = new FileReader(f);
        BufferedReader breader= new BufferedReader(reader);
        StringBuffer b = new StringBuffer();
        String read;

        while((read = breader.readLine()) != null) {
            b.append(read);
            b.append(NEW_LINE);
        }

        breader.close();
        return b.toString();
   }
   
   
    
}
