/*
 * PlasmReaderThread.java
 *
 * Created on 29 settembre 2007, 8.34
 *
 * @author Dario
 */

package it.jplasmide.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;


public class PlasmReaderThread extends Thread {
    
    private PlasmWrapper wrapper;
    
    
    public void plasmStarted(PlasmWrapper wrapper) {
       this.wrapper = wrapper;
       
       this.start();
    }
    
    public void run() {
        
        /* Set up*/
        Process p = this.wrapper.getPlasmCurrentProcess();
        
        if(p == null) {
            /* Maybe stopped, signal and stop the job */
            this.wrapper.notifyPlasmError("Process to reader thread is null");
            this.wrapper.plasmStopped();
            return;
        }
        
        /* Running */
        BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
        String read;
        
        
        // ByteBuffer behaviour
        char[] buffer = new char[1024];
        int len;
        
        try {
            
            while((len = in.read(buffer,0,1024)) > -1) {
                read = String.copyValueOf(buffer,0,len);
                System.out.println(read);
                this.wrapper.notifyPlasmCatchedOutputEvent(read);
            }
            System.out.println("EOS in reader thread");
        } catch (IOException e) {
            // Simply signal and tear down all
            System.out.println("IOException in reader thread");
        }
        
        
        /* // ReadLine behaviour
        try {
            
            while((read = in.readLine()) != null) {
                System.out.println(read);
                this.wrapper.notifyPlasmCatchedOutputEvent(read);
            }
            System.out.println("NULL in reader thread");
        } catch (IOException e) {
            // Simply signal and tear down all
            System.out.println("IOException in reader thread");
        }
        */
        
        
        /* Tear down */
        try {
            in.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        p = null;
        
        System.out.println("Reader Thread signaling...");
        this.wrapper.plasmStopped();
        this.wrapper = null;
        System.out.println("Reader Thread end...");
    }
    
}
