/*
 * PlasmWriterThread.java
 *
 * Created on 29 settembre 2007, 8.34
 *
 * @author Dario
 */

package it.jplasmide.utils;

import java.io.IOException;
import java.io.PrintStream;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;


public class PlasmWriterThread extends Thread {
    
    private PlasmWrapper wrapper;
    private BlockingQueue<String> commandsQueue = new LinkedBlockingQueue<String>();
    private String escapeSequence;
    
    public void plasmStarted(PlasmWrapper wrapper) {
       this.commandsQueue.clear();
       this.wrapper = wrapper;
       this.escapeSequence = this.wrapper.getEscapeSequence();
       
       this.start();
    }
    
    public void run() {
        
        /* Set up*/
        Process p = this.wrapper.getPlasmCurrentProcess();
        
        if(p == null) {
            /* Maybe stopped, stop the job */
            this.wrapper.notifyPlasmError("Process to writer thread is null");
            return;
        }
        
        /* Running */
        PrintStream out = new PrintStream(p.getOutputStream());
        String cmd;
        
        boolean toStop = false;
        
        while(!toStop) {

            try {

                cmd = this.commandsQueue.take();
                
                if(cmd.equals(this.escapeSequence)) {
                    toStop = true;
                    System.out.println("Escape received");
                }

                else {

                    try {
                        // New command
                        this.wrapper.notifyPlasmRequestInputEvent(PlasmWrapper.COMMAND_IN);
                        out.println(cmd);
                        out.flush();
                        // Command sent
                        this.wrapper.notifyPlasmRequestInputEvent(PlasmWrapper.COMMAND_SENT);
                        
                        this.wrapper.notifyPlasmRequestInputEvent(cmd);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                }

            } catch (InterruptedException ex) {
                ex.printStackTrace();
            }

        }
           
        
        /* Tear down */
        out.close();
        this.wrapper = null;
        p = null;
        System.out.println("Writer Thread end...");
    }
    
    public void sendCommand(String cmd) {
        if(this.wrapper != null) {
            try {
                this.commandsQueue.put(cmd);
            } catch (InterruptedException ex) {
                ex.printStackTrace();
            }
        }
    }
    
}
